package net.sf.sido.pojo;

import com.google.common.base.Function;
import net.sf.sido.DataObject;
import net.sf.sido.DataType;
import org.apache.commons.lang.ObjectUtils;

public abstract class DOObject {

    private final DOSchema schema;
    protected final DataObject o;

    public DOObject(DOSchema schema, DataObject o) {
        this.schema = schema;
        this.o = o;
    }

    public DOSchema getSchema() {
        return schema;
    }

    public <S extends DOSchema> S getSchema(Class<S> schemaClass) {
        return schema.getSchema(schemaClass);
    }

    public DataObject getDataObject() {
        return o;
    }

    public DataType getDataType() {
        return o.getSidoType();
    }

    public static Function<DOObject, DataObject> getDataObjectFn() {
        return new Function<DOObject, DataObject>() {
            @Override
            public DataObject apply(DOObject i) {
                return i.getDataObject();
            }
        };
    }

    public static <D extends DOObject> Function<D, String> getAsString(final String propertyName) {
        return new Function<D, String>() {
            @Override
            public String apply(D input) {
                Object value = input.getDataObject().get(propertyName);
                return ObjectUtils.toString(value, null);
            }
        };
    }

    @Override
    public int hashCode() {
        return o.hashCode();
    }

    @Override
    public boolean equals(Object obj) {
        if (obj instanceof DOObject) {
            return this.o.equals(((DOObject) obj).o);
        } else {
            return false;
        }
    }

    @Override
    public String toString() {
        return o.toString();
    }
}
